import { createTemplate, createRoot } from '../utils/dom.js';
const AREA = 48;
const tpl = createTemplate(`
<style>
  :host {
    display: block;
    position: relative;
    width: 48px;
    height: 48px;
    cursor: pointer;
    -webkit-user-select: none;
    user-select: none;
  }

  :host([hidden]) {
    display: none !important;
  }

  [part] {
    background: currentColor;
    position: absolute;
  }
</style>
`);
export const defaultProps = {
    size: 32,
    direction: 'left',
    distance: 'md',
    duration: 0.4,
    easing: 'cubic-bezier(0, 0, 0, 1)',
    pressed: false
};
export class Burger extends HTMLElement {
    constructor() {
        super();
        this._updatePromise = Promise.resolve();
        createRoot(this, tpl);
        this.addEventListener('click', this);
        this.addEventListener('keydown', this);
    }
    static get observedAttributes() {
        return ['distance', 'duration', 'easing', 'pressed', 'size'];
    }
    /**
     * A valid `transition-timing-function` CSS value, for example 'ease-out'.
     * @type {string}
     * @default cubic-bezier(0, 0, 0, 1)
     */
    get easing() {
        return this._easing;
    }
    set easing(easing) {
        this._easing = easing;
        this.update();
    }
    /**
     * The vertical distance between the lines. Small (sm), medium (md) or large (lg).
     * @type {'sm' | 'md' | 'lg'}
     * @default md
     */
    get distance() {
        return this._distance;
    }
    set distance(distance) {
        this._distance = distance;
        this.update();
    }
    /**
     * The duration of the animation. Can be set to zero if no animation is desired.
     * @type {number}
     * @default 0.4
     */
    get duration() {
        return this._duration;
    }
    set duration(duration) {
        this._duration = duration;
        this.update();
    }
    /**
     * Set to true when element is pressed.
     * @type {boolean}
     * @default false
     */
    get pressed() {
        return this._pressed;
    }
    set pressed(pressed) {
        this._pressed = pressed;
        this.setAttribute('aria-pressed', `${!!pressed}`);
        this.update();
    }
    /**
     * Size of the icon. Should be a number between 12 and 48.
     * @type {number}
     * @default 32
     */
    get size() {
        return this._size;
    }
    set size(size) {
        this._size = size;
        this.update();
    }
    connectedCallback() {
        this.setAttribute('role', 'button');
        this.setAttribute('tabindex', '0');
        this.constructor.observedAttributes.forEach((k) => {
            // A user may set a property on an _instance_ of an element,
            // before its prototype has been connected to this class.
            // If so, we need to run it through the proper class setter.
            if (this.hasOwnProperty(k)) {
                const value = this[k];
                delete this[k];
                this[k] = value;
            }
            else if (!this[k]) {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                this[k] = defaultProps[k];
            }
        });
    }
    attributeChangedCallback(prop, oldVal, newVal) {
        if (oldVal !== newVal) {
            let value = newVal;
            if (prop === 'size' || prop === 'distance') {
                value = newVal === null ? null : Number(newVal);
            }
            else if (prop === 'pressed') {
                value = newVal !== null;
            }
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            this[prop] = value;
        }
    }
    handleEvent(event) {
        if ((event.type === 'keydown' && event.key === 'Enter') || event.type === 'click') {
            this.pressed = !this.pressed;
            this.dispatchEvent(new CustomEvent('pressed-changed', { detail: { value: this.pressed } }));
        }
    }
    getRenderOptions() {
        const { distance, lines } = this;
        const width = Math.max(12, Math.min(AREA, this.size));
        const room = Math.round((AREA - width) / 2);
        const barHeightRaw = width / 12;
        const barHeight = Math.round(barHeightRaw);
        const space = distance === 'lg' ? 0.25 : distance === 'sm' ? 0.75 : 0.5;
        const marginRaw = width / (lines * (space + (lines === 3 ? 1 : 1.25)));
        const margin = Math.round(marginRaw);
        const height = barHeight * lines + margin * (lines - 1);
        const topOffset = Math.round((AREA - height) / 2);
        const translate = lines === 3
            ? distance === 'lg'
                ? 4.0425
                : distance === 'sm'
                    ? 5.1625
                    : 4.6325
            : distance === 'lg'
                ? 6.7875
                : distance === 'sm'
                    ? 8.4875
                    : 7.6675;
        const deviation = (barHeightRaw - barHeight + (marginRaw - margin)) / (lines === 3 ? 1 : 2);
        const move = parseFloat((width / translate - deviation / (4 / 3)).toFixed(2));
        const barStyles = {
            height: `${barHeight}px`,
            left: `${room}px`,
            width: `${width}px`
        };
        return {
            barHeight,
            barStyles,
            margin,
            move,
            pressed: this.pressed,
            time: Math.max(0, this.duration),
            topOffset
        };
    }
    update() {
        if (!this._updating) {
            this._updatePromise = this._enqueueUpdate();
        }
    }
    async _enqueueUpdate() {
        this._updating = true;
        // Ensure that property changes are batched.
        await this._updatePromise;
        this.render(this.getRenderOptions());
        this._updating = false;
    }
}
//# sourceMappingURL=burger.js.map